@@ -287,6 +287,7 @@ def create_pkginfo_from_as_pkg(as_pkg, remote_name, remote_url):
 
     pkg_hash = make_pkg_hash(ref)
     pkginfo = FlatpakPkgInfo(pkg_hash, remote_name, ref, remote_url)
+    pkginfo.add_cached_appstream_data(as_pkg)
     pkginfo.installed = isinstance(ref, Flatpak.InstalledRef)
 
     return pkginfo
@@ -370,11 +371,10 @@ def _get_related_refs_for_removal(parent_pkginfo):
                                                                  None)
     return related_refs
 
-def _get_addon_refs_for_pkginfo(parent_pkginfo):
+def _get_addons_for_pkginfo(parent_pkginfo):
     global pools
 
-    addon_refs = []
-
+    matched_addons = []
     try:
         aspool = pools[parent_pkginfo.remote]
         as_pkg = aspool.lookup_appstream_package(parent_pkginfo)
@@ -385,11 +385,84 @@ def _get_addon_refs_for_pkginfo(parent_pkginfo):
             for addon in addons:
                 info = create_pkginfo_from_as_pkg(addon, parent_pkginfo.remote, parent_pkginfo.remote_url)
                 if info:
-                    addon_refs.append(info.refid)
+                    if _addon_is_compatible(parent_pkginfo, info):
+                        matched_addons.append(info)
     except Exception as e:
-        debug(str(e))
+        warn("Could not get a list of addons: %s" % str(e))
 
-    return addon_refs
+    return matched_addons
+
+def _get_metadata(remote_name, ref):
+    try:
+        # RemoteRef
+        meta = ref.get_metadata()
+    except AttributeError:
+        meta = get_fp_sys().fetch_remote_metadata_sync(remote_name, ref, None)
+
+    data = meta.get_data().decode()
+
+    keyfile = GLib.KeyFile.new()
+    keyfile.load_from_data(data, len(data), GLib.KeyFileFlags.NONE)
+
+    return keyfile
+
+def _addon_is_compatible(parent, addon):
+    # Get the extension point name
+    parent_meta = _get_metadata(parent.remote, Flatpak.Ref.parse(parent.refid))
+    child_meta = _get_metadata(addon.remote, Flatpak.Ref.parse(addon.refid))
+
+    # When multiple extensions of the same type can be used, the
+    # addon's ID will have the prefix of its intended extension point:
+    # org.gimp.GIMP.Plugin.BIMP -> org.gimp.GIMP.Plugin.
+    #
+    # Plugins built with the primary package will have their full name as
+    # the extension group.
+    addon_prefix = addon.name.rpartition(".")[0]
+    ext_point = f"Extension {addon_prefix}"
+
+    # Addons should always have a 'ref' field, at minimum, to match them with their app.
+    try:
+        eo_ref = child_meta.get_string("ExtensionOf", "ref")
+        if eo_ref != parent.refid:
+            return False
+    except:
+        pass
+
+    groups, l = parent_meta.get_groups()
+
+    for group in groups:
+        # skip irrelevant groups
+        if not group.startswith("Extension "):
+            continue
+        if group not in (ext_point, f"Extension {addon.name}"):
+            continue
+
+        # Look for a version field, see if it matches the addon's branch
+        versions = []
+        try:
+            versions = parent_meta.get_string_list(group, "versions")
+        except GLib.Error as e:
+            try:
+                versions = [parent_meta.get_string(group, "version")]
+            except:
+                pass
+        if len(versions) > 0:
+            return addon.branch in versions
+
+        # See if the extension specifies a runtime, and if it matches the app's.
+        # This may end up filtering out some valid addons if no extension versioning
+        # is used, but...
+        try:
+            child_runtime = child_meta.get_string("ExtensionOf", "runtime")
+            parent_runtime = parent_meta.get_string("Application", "runtime")
+
+            if child_runtime != parent_runtime:
+                return False
+        except GLib.Error as e:
+            pass
+
+    # All else fails, let it thru anyhow. Who knows? Do you??
+    return True
 
 def select_packages(task):
     task.transaction = FlatpakTransaction(task)
@@ -443,9 +516,9 @@ class FlatpakTransaction():
                 if not self.task.is_addon_task:
                     for related_ref in _get_related_refs_for_removal(self.task.pkginfo):
                         self.transaction.add_uninstall(related_ref.format_ref())
-                    for addon_formatted_ref in _get_addon_refs_for_pkginfo(self.task.pkginfo):
+                    for addon_info in _get_addons_for_pkginfo(self.task.pkginfo):
                         try:
-                            self.transaction.add_uninstall(addon_formatted_ref)
+                            self.transaction.add_uninstall(addon_info.refid)
                         except GLib.Error as e:
                             if e.code != Flatpak.Error.NOT_INSTALLED:
                                 warn("Could not add uninstall for addon '%s': %s" % (addon_formatted_ref, e.message))
@@ -578,6 +651,9 @@ class FlatpakTransaction():
         if error.code == Gio.IOErrorEnum.CANCELLED:
             return False
 
+        if self.task.type == self.task.UNINSTALL_TASK and error.code == Flatpak.Error.NOT_INSTALLED:
+            return True
+
         self.op_error = error
         self.log_operation_result(operation, None, error)
 
@@ -715,22 +791,6 @@ class FlatpakTransaction():
 
         ref_list.append(ref)
 
-    def _get_runtime_ref_from_remote_metadata(self, remote_name, ref_str):
-        runtime_ref = None
-
-        ref = Flatpak.Ref.parse(ref_str)
-
-        meta = get_fp_sys().fetch_remote_metadata_sync(remote_name, ref, None)
-        data = meta.get_data().decode()
-
-        keyfile = GLib.KeyFile.new()
-        keyfile.load_from_data(data, len(data), GLib.KeyFileFlags.NONE)
-
-        runtime = keyfile.get_string("Application", "runtime")
-        runtime_ref = Flatpak.Ref.parse("runtime/%s" % runtime)
-
-        return runtime_ref.format_ref()
-
     def _confirm_transaction(self):
         # only show a confirmation if:
         # - (install/remove) Additional changes are triggered for more than just the selected package.
